@@ -54,7 +54,7 @@ static const char *fix_mcwd(char *ans)
 					/* drive letter present? */
 	s = buf;
 	if (buf[0] && buf[1] == ':') {
-		strncpy(ans, buf, 2);
+		memcpy(ans, buf, 2);
 		ans[2] = '\0';
 		s = &buf[2];
 	} else {
@@ -71,7 +71,7 @@ static const char *fix_mcwd(char *ans)
 #if 0
 					/* translate to upper case */
 	for (s = ans; *s; ++s) {
-		*s = toupper(*s);
+		*s = ch_toupper(*s);
 		if (*s == '\\')
 			*s = '/';
 	}
@@ -86,7 +86,7 @@ static const char *fix_mcwd(char *ans)
 }
 
 int unix_dir_loop(Stream_t *Stream, MainParam_t *mp); 
-int unix_loop(UNUSED(Stream_t *Stream), MainParam_t *mp, char *arg,
+int unix_loop(Stream_t *Stream UNUSEDP, MainParam_t *mp, char *arg,
 	      int follow_dir_link);
 
 static int _unix_loop(Stream_t *Dir, MainParam_t *mp,
@@ -95,11 +95,11 @@ static int _unix_loop(Stream_t *Dir, MainParam_t *mp,
 	return unix_dir_loop(Dir, mp);
 }
 
-int unix_loop(UNUSED(Stream_t *Stream), MainParam_t *mp,
+int unix_loop(Stream_t *Stream UNUSEDP, MainParam_t *mp,
 	      char *arg, int follow_dir_link)
 {
 	int ret;
-	int isdir;
+	int isdir=0;
 	int unixNameLength;
 
 	mp->File = NULL;
@@ -223,7 +223,7 @@ static int handle_leaf(direntry_t *direntry, MainParam_t *mp,
 				return 0;
 			case 1: /* we have already a directory */
 				FREE(&lookupState->Dir);
-				fprintf(stderr,"Ambigous\n");
+				fprintf(stderr,"Ambiguous\n");
 				return STOP_NOW | ERROR_ONE;
 			default:
 				return STOP_NOW | ERROR_ONE;
@@ -258,8 +258,9 @@ static int _dos_loop(Stream_t *Dir, MainParam_t *mp, const char *filename)
 	initializeDirentry(&entry, Dir);
 	while(!got_signal &&
 	      (r=vfat_lookup(&entry, filename, -1,
-			     mp->lookupflags, mp->shortname,
-			     mp->longname)) == 0 ){
+			     mp->lookupflags,
+			     mp->shortname.data, mp->shortname.len,
+			     mp->longname.data, mp->longname.len)) == 0 ){
 		mp->File = NULL;
 		if(!checkForDot(mp->lookupflags,entry.name)) {
 			MyFile = 0;
@@ -390,7 +391,8 @@ static int recurs_dos_loop(MainParam_t *mp, const char *filename0,
 	      !got_signal &&
 	      (r=vfat_lookup(&entry, filename0, length,
 			     lookupflags | NO_MSG,
-			     mp->shortname, mp->longname)) == 0 ){
+			     mp->shortname.data, mp->shortname.len,
+			     mp->longname.data, mp->longname.len)) == 0 ){
 		if(checkForDot(lookupflags, entry.name))
 			/* while following the path, ignore the
 			 * special entries if they were not
@@ -433,7 +435,7 @@ static int common_dos_loop(MainParam_t *mp, const char *pathname,
 	drive='\0';
 	cwd = "";
 	if(*pathname && pathname[1] == ':') {
-		drive = toupper(*pathname);
+		drive = ch_toupper(*pathname);
 		pathname += 2;
 		if(mp->mcwd[0] == drive)
 			cwd = mp->mcwd+2;
@@ -504,7 +506,7 @@ static int dos_target_lookup(MainParam_t *mp, const char *arg)
 			return ret;
 		default:
 			/* too much */
-			fprintf(stderr, "Ambigous %s\n", arg);
+			fprintf(stderr, "Ambiguous %s\n", arg);
 			return ERROR_ONE;			
 	}
 }
@@ -602,7 +604,8 @@ void init_mp(MainParam_t *mp)
 	mp->unixTarget = 0;
 	mp->dirCallback = dispatchToFile;
 	mp->unixcallback = NULL;
-	mp->shortname = mp->longname = 0;
+	mp->shortname.data = mp->longname.data = 0;
+	mp->shortname.len = mp->longname.len = 0;
 	mp->File = 0;
 	mp->fast_quit = 0;
 }
@@ -611,7 +614,7 @@ const char *mpGetBasename(MainParam_t *mp)
 {
 	if(mp->direntry) {
 		wchar_to_native(mp->direntry->name, mp->targetBuffer,
-				MAX_VNAMELEN+1);
+				MAX_VNAMELEN+1, sizeof(mp->targetBuffer));
 		return mp->targetBuffer;
 	} else
 		return _basename(mp->unixSourceName);
